#!/usr/bin/python
# -*- coding: UTF-8 -*-

# 26.09.2014
#
# --- [ input_row.py ] ---------------------------------------------------------
#
# Hier ist eine einzelne Eingabezeile als Klasse definiert

import tkinter as tk
from tkinter import ttk

class Input_row(object):
    """
    Stellt eine Eingabezeile bereit, in der vier Werte eingegeben, editiert und
    abgesendet werden können.
    Die manuelle Eingabe wird validiert.
    """
    idx = 0

    def __init__(self, frm,
                 column   = 0,
                 row      = 0,
                 parent   = None,
                 ser      = None,
                 callback = None ):

        self.ser      = ser
        self.idx      = Input_row.idx
        Input_row.idx += 1
        self.row      = row - 2
        self.parent   = parent
        self.callback = callback
        self.appName  = "IRMP-rx"

                # Bezeichner, Spalte, Textbreite, [Entry->] Spalte, EingabeBreite
        items = (   ("IR-Protokoll:", 0, 13, 1,  2),
                    ("Adr:",          2,  5, 3,  4),
                    ("Cmd:",          4,  5, 5,  4),
                    ("Memo:",         7,  6, 8, 10) )

        self.var    = []
        self.entrys = []

        for i in range(len(items)):
            self.var.append(tk.StringVar())
            ttk.Label(frm, text = items[i][0], width = items[i][2], anchor = "e").grid(
                    column = items[i][1], row = row, padx = 5, pady = 0, sticky = "nsew")

            self.entrys.append( ttk.Entry(frm, width = items[i][4], \
                                textvariable = self.var[i], \
                                font = ('Courier', 10, 'bold' )))
            self.entrys[i].grid(column = items[i][3], row = row, padx = 5, pady = 5, sticky = "nswe")

            # Das Protokoll wird durch eine 2 stellige Dezimalzahl dargestellt
            if i == 0:
                self.entrys[i].configure(validate = 'key',
                    validatecommand = (self.entrys[i].register(self.isDez99), '%P'))

            # Adresse / Command werden 4 stellige Hexadezimalzahlen dargestellt
            # Memo (3) ist ein reines Textfeld
            elif i < 3:
                self.entrys[i].configure(validate = 'key',
                    validatecommand = (self.entrys[i].register(self.isHex16), '%P'))


            #self.entrys[i].bind(sequence ='<Return>', func= self.__send)

        ttk.Button(frm, width = 2, text = "<", command = self.onGetValues ).\
                grid(column = 6, row = row, padx = 5, pady = 5, sticky = "nse")

        ttk.Button(frm, width = 12, text = "senden", command = self.send ).\
                grid(column = 9, row = row, padx = 5, pady = 5, sticky = "nsew")


    def isDez99(self, new_value):
        """ Testet auf: Dezimalzahl mit maximal 2 Stellen """
        if len(new_value) > 2:
            return False

        if not new_value:
            return True

        try:            # hier die Prüfung, die gelingen muss
            i = int(new_value, 10)
            if i < 0:
                raise BaseError
        except:         # wenn nicht, dann wird die Eingabe rückgängig gemacht
            return False

        self.parent.modified = True
        return True


    def isHex16(self, new_value):
        """ Testet auf: Hexadezimalzahl mit bis zu 4 Stellen """
        if len(new_value) > 4:
            return False

        if not new_value:
            return True

        try:            # hier die Prüfung, die gelingen muss
            i = int(new_value, 16)
            if i < 0:
                raise BaseError
        except:         # wenn nicht, dann wird die Eingabe rückgängig gemacht
            return False
        self.parent.modified = True
        return True


    def onGetValues(self):
        """
        Holt indirekt die ausgewählte Zeile aus dem Editor ab
        der Inhalt wird via self.setSelection geschrieben.
        """
        self.callback(self.idx)


    def setSelection(self, data):
        """
        Schreibt die in main() geprüften Daten in die Eingabezeile
        """
        for i in range(3):
            self.var[i].set(data[i])


    def send(self, event = None):
        """
        Wird ausgeführt, wenn der Button "absenden" gedrückt wird.
        Wandelt die Eingabefelder in ein Bytearray um, das via
        serieller Schnittstelle versandt werden kann.
        """

        # die zu sendenden Daten beginnen mit der Konstanten 0x7E
        cmd = [0x7E]
        error = False

        for i in range(3):
            try:
                if i == 0:
                    base = 10
                else:
                    base = 16
                x = int(self.var[i].get(), base)
                if x < 0:
                    raise BaseError

            except:
                error = True
                x = 0
                tk.messagebox.showwarning(self.appName, \
                    "Ungültige Eingabe in Feld {}\n'{}'\nist kein UINT mit Base {} !".\
                    format( i + 1, self.var[i].get(), base))
                break

            if i == 0:                  # 8 Bit
                cmd.append(x & 0xFF)

            else:                       # 16 Bit
                x &= 0xFFFF
                cmd.append((x >> 8))
                cmd.append((x & 0xFF))

        if error is False:
#            print(cmd)
            self.ser.write(cmd)


    def getData(self):
        """ Die Daten auslesen zum pickeln """
        data = []
        for i in range(4):
            data.append(self.var[i].get())
        return data


    def putData(self, data):
        """ Gepickelte Daten wieder in die Felder schreiben """
        for i in range(4):
            self.var[i].set(data[i])
