#!/usr/bin/python
# -*- coding: UTF-8 -*-
#
# 26.09.2014
#
# --- [ irmp_main.py ] -------------------------------------------


import tkinter as tk
#from tkinter import Font
from tkinter import filedialog
from tkinter import ttk
from tkinter import messagebox
from irmp_fdc import Irmp_FDC
import pickle, os
import sys, time

RECEIVE_ROWS = 15                   # Anzahl der Zeilen im ReceiveFrame
SEND_ROWS    =  6                   # Anzahl der Zeilen im SendFrame
DEFAULT_INI_FILE = "default.pic"    # Name der Datei mit den default-Einstellungen


from irmp_rx import Irmp_rx
from serport import open_serport
from simple_editor import Editor
from input_row import Input_row
from irmp_help import helpScreen, helpGet


class IrmpWindow(tk.Tk):
    def __init__(self, ser):
        tk.Tk.__init__(self)
        self.appName = "IRMP / IRSND"
        self.title(self.appName)
        self.resizable(width = False, height = False)
        self.protocol('WM_DELETE_WINDOW', self._quit)

        #print(self.winfo_screenwidth())
        # bei einem Netbook die Höhe des Editorfenster reduzieren
        if int(self.winfo_screenheight()) < 700:
            global RECEIVE_ROWS
            if RECEIVE_ROWS > 10:
                RECEIVE_ROWS = 10
            self.geometry("+100+50")
        else:
            self.geometry("+200+100")

        s = ttk.Style()
#        print(s.theme_names())
#        print(s.theme_use())
        s.configure('.', font =('Helvetica', 8, 'bold'))

        self.bind('<Escape>', self._quit)
        self.bind('<F3>', self.help)

        self.ser       = ser
        self.selection = ""
        self.modified  = False
        self.row_cnt   = 0

        self.e = Editor(self, columnspan = 10, width = 80, height = RECEIVE_ROWS)
        self.e.bind('<1>', self.onMouse1)
        self.e.bind('<3>', self.onMouse3)
        # hier wird die Farbe festgelegt, mit der eine selektierte Zeile markiert wird
        # ein helles Mausgrau
        self.e.tag_config('my', background = "#DDDDDD")

        self.z = []
        for i in range(SEND_ROWS):
            self.z.append(Input_row(self, parent = self, row = i + 2, ser = self.ser,
                                                callback = self.onCopyRow))
        self.rx = Irmp_rx(ser = self.ser, callback = self.cbReceived)
        self.rx.start()

#        self.fdc = Irmp_FDC()

        # gespeicherte Einstellungen laden - falls vorhanden
        self.onOpenDefault()

        self.addMenus()

        self.mainloop()
        self.destroy()


    def addMenus(self):
        menus = (
                ("Datei", 0,[
                    ("Als Default speichern",          4, self.onSave),
                    ("---"),
                    ("Einstellungen speichern unter", 14, self.onSaveAs),
                    ("Einstellungen laden",           14, self.onOpenFile),
                    ("---"),
                    ("Beenden",                        0, self._quit)]),
                    )

        menubar = tk.Menu(self)
        self.config(menu = menubar)
        for name, underline, items in menus:
            pulldown = tk.Menu(menubar, tearoff=False)
            self.addMenuItems(pulldown, items)
            menubar.add_cascade(label = name, underline = underline, menu = pulldown)


    def addMenuItems(self, menu, items):
        for item in items:
            if item == "---":
                menu.add_separator()
            elif type(item) != list:
                menu.add_command(label = item[0],
                                 underline = item[1],
                                 command = item[2])


    def onSaveAs(self):
        """ Die Einstellungen unter einem neuen Namen speichern """
        file = tk.filedialog.asksaveasfilename(defaultextension = '.pic',\
                    title = "Einstellungen speichern",\
                    initialfile = DEFAULT_INI_FILE.split('.')[0], \
                    filetypes =[("PIC", "*.pic"),
                                ("alle Dateien", "*.*")] )
        if file:
            self.onSave(file = file)


    def onOpenFile(self):
        """ Fragt nach einem Dateinamen vor dem Einlesen der Einstellungen """
        file = tk.filedialog.askopenfilename(defaultextension = '.pic',\
                    title = "Einstellungen laden",\
                    initialfile = DEFAULT_INI_FILE.split('.')[0],\
                    filetypes =[("PIC", "*.pic"),
                                ("alle Dateien", "*.*")] )
        if file:
            self.onOpenDefault(file = file)


    def cbReceived(self, header, data, zeit):
        """
        Callback zur Aktualisierung der Anzeige,
        eingehende Daten werden als neue Zeile im Editor ausgegeben.

        Damit nicht irgendwelche zufälligen Daten entgegengenommen werden,
        wird ein Protokoll vereinbart:

        Der AVR muss senden: 1 StartByte, 5 Datenbytes

        die Zeit steuert der Empfangsthread bei.
        """

        if header == 0x7E:
            # IR-Tastatur, Protokoll 18
##            if data[0] == 18:
##                t = self.fdc.get_chr(data[3] * 256 + data[4])
##                if t is not None:
##                    self.e.insert('end', t)
##                return

            t = "{:03}  P: {:02} - Adr: {:04X}, Cmd: {:04X}, Flags: {:04b} - time: {}".\
                        format(
                        self.row_cnt, data[0],
                        data[1] * 256 + data[2],
                        data[3] * 256 + data[4],
                        data[5], zeit)

        elif header == 0xE7:
            self.bell()
            t = "Fehler beim Versenden von Protokoll {:02} -> {}".\
                        format(data[0], helpGet('{:02}'.format(data[0])))

        else:
            self.bell()
            t = "Unbekannter Header empfangen"

        self.e.insert('end', t +"\n")
        self.e.see('end')

        self.row_cnt += 1
        if self.row_cnt > 999:
            self.row_cnt = 0


    def onCopyRow(self, row):
        """
        Der Copy-Button in der Eingabezeile 'row' wurde gedrückt.
        Der Inhalt der Zeile steht in 'self.selection'
        diese Zeile auf Plausibilität prüfen und bei Erfolg an die
        Eingabezeile weiterreichen
        """
        if self.selection:
            if self.selection[5:8] !=  'P: ':
                tk.messagebox.showwarning(self.appName, \
                    "Diese Zeile enthält keine Protokolldaten !")
                return

            # die Eingabefelder prüfen
            tmp = (self.selection[8:10], self.selection[18:22], self.selection[29:33])
            error = False
            for i in range(3):
                try:
                    if i == 0:
                        base = 10
                    else:
                        base = 16
                    x = int(tmp[i], base)
                    if x < 0:
                        raise BaseError
                    # falls manuell im Editor ein 'x' eingemogelt worden ist
                    if (tmp[i].upper()).find('X') > -1:
                        raise BaseError

                except:
                    error = True
                    txt = ("Protokoll", "Adresse", "Command")[i]
                    tk.messagebox.showwarning(self.appName, \
                        "Ungültige Daten in ausgewählter Zeile:\n\n{}\n{}: -> '{}'".\
                            format(self.selection, txt, tmp[i]))
                    break

            if error is False:
                self.z[row].setSelection(tmp)
                self.modified = True

        else:
            tk.messagebox.showwarning(self.appName, \
                    "Es sind keine Daten ausgewählt")


    def onMouse1(self, e):
        """ Die linke Maustaste löscht jede Auswahl im Tag 'my' """
        self.e.tag_remove("my", '1.0', tk.END)
        self.selection = ""


    def onMouse3(self, e):
        """
        Die rechte Maustaste markiert die gesamte Zeile und kopiert
        den Inhalt der Zeile in die Variable 'self.selektion'
        """
        start = self.e.index(tk.CURRENT).split('.')[0] + '.0'
#        end   = start + 'lineend'
        end   = start + '+1l'

        # den Bereich löschen, so dass immer nur eine einzige Zeile enthalten ist
        self.e.tag_remove("my", '1.0', tk.END)
        self.e.tag_add("my", start, end)
        # den Inhalt der Zeile in eine Variable kopieren
        self.selection = self.e.get(start, end)


    def help(self, e = None):
        """
        Die Hilfe greift auf die Zeichenfolge hinter 'P: ' zurück,
        Hier muss eine Zahl stehen.
        """
        if self.selection and self.selection[5:8] == "P: ":
            try:
                a = int(self.selection[8:10])
            except:
                tk.messagebox.showwarning(self.appName,
                        "Ungültiger Protokolleintrag !")
            else:
                helpScreen(self.selection[8:10])
        else:
            tk.messagebox.showwarning(self.appName,
                        "Es ist keine gültige Protokollzeile selektiert !")


    def _quit(self, e = None):
        """ Wenn das App-Window mit dem Button 'X' oben rechts geschlossen wird
            muss der Hintergrundthread beendet und die serielle Schnittstelle
            noch geschlossen werden
        """
        if self.modified:
            r =tk.messagebox.askyesnocancel(self.appName, \
                "Es wurden Einstellung geändert !\nÄnderungen speichern ?")
            if r is None:
                return
            elif r:
                self.onSaveAs()

        self.rx.exit()
        self.ser.close()
        self.quit()


    def onSave(self, file = DEFAULT_INI_FILE):
        """ Schreibt die aktuellen Einstellungen in eine Datei """
        values = []
        for z in self.z:
            values.append(z.getData())

#        print(values)

        f = open(file, "wb")
        pickle.dump(values, f)
        f.close()


    def onOpenDefault(self, file = None):
        """ Öffnet die Default-Datei - oder eine andere """

        if file is None:
            if os.path.isfile(DEFAULT_INI_FILE):
                f = open(DEFAULT_INI_FILE, "rb")
            else:
                return

        elif os.path.isfile(file):
            f = open(file, "rb")

        else:
            messagebox.showwarning(self.appName, \
                    "Datei {} nicht gefunden !".format(file))
            return


        idx = 0
        try:
            values = pickle.load(f)
            f.close()
#            print(values)

            i = 0
            for z in self.z:
                z.putData(values[i])
                i+= 1

        except:
            messagebox.showwarning(self.appName, \
                    "Fehler beim Einlesen von Einstellung\n"
                    + "aus der Datei {}!".format(file))


if __name__ == "__main__":

    ser = open_serport(start = 1, end = 13)
    IrmpWindow(ser)

# --- [ eof ] ----------------------------------------------------
