#! /usr/bin/python3
# -*- coding: UTF-8 -*-
#
#       26.01.2014
#
# --- [ irmp_rx.py ] -----------------------------------------------------------
#
# Ist für das Empfangen von Frames via Seriellen Schnittstelle zuständig
# liefert zurück:
# - das Startbyte
# - den Frame
# - die Zeit des Empfangs

import serial
import threading
import time

class Irmp_rx(threading.Thread):
    """
    Empfängt einen Frame mit den Daten:
    1.) Startbyte 0x7E (oder 0xE7 bei einem Fehler)
    2.) Code des IR-Protokolls
    3.) 16Bit Adresse
    4.) 16Bit Kommando
    5.) 8Bit Flag
    """
    def __init__(self, ser, callback = None):
        threading.Thread.__init__(self)
        self.ser = ser
        self.cb = callback
        self.threadend = False


    def exit(self):
        """ Beendet den Thread (mit kurzer Verzögerung durch die sleep's) """
        self.threadend = True
        #print("Thread Irmp_rx quit")


    def run(self):
        """
        Startet den Hintergrundthread, der die Daten von der Seriellen
        Schnittstelle abholt.

        Rückgabe per Callback:
        - Datenframe als Bytestring,
        - Zeitpunkt des Empfangs als String.
        """
        self.ser.flush()                        # alle anstehenden Daten löschen
#        print("Thread rx started")

        while self.threadend is False:
            i = self.ser.inWaiting()

            if i is 0:
                time.sleep(0.1)                 # 100ms warten

            else:
                # die Systemzeit mit 1 Nachkommastelle festhalten
                timestring = (time.ctime()[11:19]+ "." +
                                                str(round(time.time(),1))[-1:])

                frm_head = self.ser.read(1)  # das erste Byte einlesen

#                print(frm_head)

                # Wenn die ersten beiden Bytes nicht mit dem erwarteten Beginn eines Frames übereinstimmmen.
                # dann wurde kein Framestart erkannt, wir sind mitten in einem Frame gelandet.
                # Warten bis zum Ende der Übertragung und alle Daten in der Zwischenzeit verwerfen.
                head_error = False
                if ( not ((frm_head[0] == 0x7E) or (frm_head[0] == 0xE7))):
                    while True:
                        # 10 Millisekunden, sollten für mindestens 10 Zeichen ausreichen
                        time.sleep(.01)

                        i = self.ser.inWaiting()
                        if i == 0: # wenn kein Byte eingegangen,
                            head_error = True
                            break           # dann kann die Schleife verlassen werden

                        print("inWaiting: header: {:02X}, cnt: {:02X} data: {}".format(
                                frm_head[0], i, self.ser.read(i)))


                # Warten bis das Frame vollständig im Empfangspuffer vorliegt
                # es werden genau 6 Byte erwartet.
                # Abbrechen, falls innerhalb von 12 ms weniger Bytes eintreffen
                if head_error == False:
                    if frm_head[0] == 0x7E:
                        cnts = 6

                    elif frm_head[0] == 0xE7:
                        cnts = 1

                    else:
                        print("Error in Header: {:02X}". format(frm_head[0]))
                        exit()

                    wait = 0
                    while (self.ser.inWaiting() < cnts):
                        time.sleep(.005)
                        wait += 1
                        if wait > 3:            # beenden, wenn keine Daten
                            self.ser.read(self.ser.inWaiting())
                            print("time overflow")
                            break

                    i = self.ser.inWaiting()

                    if i > (cnts - 1):
                        frm_data = self.ser.read(cnts)

#                        print(frm_head[0], frm_data, timestring )

                        self.cb(frm_head[0], frm_data, timestring )

                    else:
                        print("rx err:", self.ser.read(i))

                continue

#        print("Thread rx closed")
        exit()

# --- [ eof ] ------------------------------------------------------------------
