/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * main.c - demo main module to test irmp decoder
 *
 * Copyright (c) 2009-2013 Frank Meyer - frank(at)fli4l.de
 *
 * $Id: main.c,v 1.17 2013/01/17 07:33:14 fm Exp $
 *
 * This demo module is runnable on AVRs and LM4F120 Launchpad (ARM Cortex M4)
 *
 * ATMEGA88 @ 8 MHz internal RC      Osc with BODLEVEL 4.3V: lfuse: 0xE2 hfuse: 0xDC efuse: 0xF9
 * ATMEGA88 @ 8 MHz external Crystal Osc with BODLEVEL 4.3V: lfuse: 0xFF hfuse: 0xDC efuse: 0xF9
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */

#include "irmp.h"
#include "irsnd.h"
#include "rs232.h"
#include <util/delay.h>

#ifndef F_CPU
#error F_CPU unkown
#endif

#define FDC_KEY

#ifdef FDC_KEY
extern uint8_t get_fdc_key (uint16_t cmd);
#endif

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * ATMEL AVR part:
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
#if defined (ATMEL_AVR)

void
timer1_init (void)
{
#if defined (__AVR_ATtiny45__) || defined (__AVR_ATtiny85__)                // ATtiny45 / ATtiny85:

#if F_CPU >= 16000000L
    OCR1C   =  (F_CPU / F_INTERRUPTS / 8) - 1;                              // compare value: 1/15000 of CPU frequency, presc = 8
    TCCR1   = (1 << CTC1) | (1 << CS12);                                    // switch CTC Mode on, set prescaler to 8
#else
    OCR1C   =  (F_CPU / F_INTERRUPTS / 4) - 1;                              // compare value: 1/15000 of CPU frequency, presc = 4
    TCCR1   = (1 << CTC1) | (1 << CS11) | (1 << CS10);                      // switch CTC Mode on, set prescaler to 4
#endif

#else                                                                       // ATmegaXX:
    OCR1A   =  (F_CPU / F_INTERRUPTS) - 1;                                  // compare value: 1/15000 of CPU frequency
    TCCR1B  = (1 << WGM12) | (1 << CS10);                                   // switch CTC Mode on, set prescaler to 1
#endif

#ifdef TIMSK1
    TIMSK1  = 1 << OCIE1A;                                                  // OCIE1A: Interrupt by timer compare
#else
    TIMSK   = 1 << OCIE1A;                                                  // OCIE1A: Interrupt by timer compare
#endif
}

#ifdef TIM1_COMPA_vect                                                      // ATtiny84
#define COMPA_VECT  TIM1_COMPA_vect
#else
#define COMPA_VECT  TIMER1_COMPA_vect                                       // ATmega
#endif

ISR(COMPA_VECT)                                                             // Timer1 output compare A interrupt service routine, called every 1/15000 sec
{
  if (!irsnd_ISR())
      {
      (void) irmp_ISR();                                                     // call irmp ISR
      }
  // call other timer interrupt routines...
}

int main (void)
{
	uint8_t i, in;
	uint8_t rx[8];

    IRMP_DATA irmp_data;

    irmp_init();                        // initialize irmp
    irsnd_init();                       // initialize irsnd
    timer1_init();                      // initialize timer1
	rs232_init();						// initialize rs232
    sei();                              // enable interrupts


    // Irgendetwas senden als Test der seriellen Verbindung
    put_c(0x7E);
    for (i= 0; i < 7; i++) put_c(i);

    for (;;)
    {
        if (irmp_get_data (&irmp_data))
        {
            // StartByte '0x7E' + 6 DatenBytes senden
            // 0x7E, Protokoll_8, Adresse_16, Command_16, Flags_8 -> 7 Byte
            put_c(0x7E);
            put_c(irmp_data.protocol);
            put_c((uint8_t) (irmp_data.address >> 8));
            put_c((uint8_t)  irmp_data.address);
            put_c((uint8_t) (irmp_data.command >> 8));
            put_c((uint8_t)  irmp_data.command);
            put_c(irmp_data.flags);
        }

        // prfen, ob via Serieller Schnittstelle Daten eingegangen sind
        if (bytes_received())
        {
            // bis zu 12ms warten bis alle ewarteten 6 Bytes eingetroffen sind,
            // werden weniger Bytes gesendet, dann wird die Schleife nach
            // 12ms verlassen.
            // 0x7E, Protokoll_8, Adresse_16, Command_16 = 6 Byte
            i = 12;
            while (i)
            {
                i--;
                _delay_ms(1);
                in = bytes_received();
                if (in >= 6) break;
            }

            if (in == 6)
            {
                get_bytes(rx);
                // Das erste Byte auf 0x7E prfen
                if (rx[0] == 0x7E)
                {
                    irmp_data.protocol = rx[1];
                    irmp_data.address  = ((rx[2] << 8) + rx[3]);
                    irmp_data.command  = ((rx[4] << 8) + rx[5]);
                    irmp_data.flags    = 0;

                    // Return -> 0 == Fehler, -> 1 == Erfolg,
                    if (!(irsnd_send_data (&irmp_data, TRUE)))
                    {
                        // FehlerByte '0xE7' + Protokollnummer senden
                        put_c(0xE7);
                        put_c(irmp_data.protocol);
                    }
                }
            }
        }
    }
}

#endif
