// --- [ rs232.c ] ------------------------------------------------------------

#include <avr/io.h>
#include <avr/interrupt.h>
#include "rs232.h"

uint8_t rx_buffer[RX_BUFFER];
volatile uint8_t rx_head;

// ----------------------------------------------------------------------------
//
void rs232_init(void)
{
    UBRRL = TEILER;
    UBRRH = 0;
    UCSRB |= (1<<RXCIE | 1<<TXEN | 1<<RXEN);
    UCSRC |= (1<<URSEL | 0<<UMSEL) | (1<<UCSZ1) | (1<<UCSZ0);
    // URSEL - Achtung bei ATmega 8
    //UCSRC |= (0<<UMSEL) | (1<<UCSZ1) | (1<<UCSZ0);
}

// ----------------------------------------------------------------------------
// ein einzelnes Zeichen auf COM ausgeben
void put_c(uint8_t Zeichen)
{
    while(!(UCSRA & (1<<UDRE)));
    UDR = Zeichen;
}

// ----------------------------------------------------------------------------
ISR(USART_RXC_vect)
// wird aufgerufen, wenn ein Byte via RS232 eingegangen ist.
// Das Byte wird im Empfangspuffer abgelegt,
// der Index incrementiert und auf den Maximalwert begrenzt.
{
   rx_buffer[rx_head++] = UDR;
   rx_head &= RX_BUFFER_MASK;
}

// ----------------------------------------------------------------------------
uint8_t bytes_received(void)
// liefert zurck, wieviele Bytes im Empfangspuffer sind
{
    return rx_head;
}

// ----------------------------------------------------------------------------
void get_bytes(uint8_t * data)
// holt die Daten aus dem Empfangspuffer ab und setzt
// den Index auf 0 zurck
{
   uint8_t i;
   for (i = 0; i <rx_head; i++)
   {
         data[i] = rx_buffer[i];
   }
   rx_head = 0;
}

// --- [ eof ] ----------------------------------------------------------------
