#!/usr/bin/python3
# -*- coding: UTF-8 -*-
#       21.02.2014
#
# --- [ serdialog.py ] ---------------------------------------------------------
#
# Dialog zur Auswahl einer Seriellen Schnittstelle
#
# Öffnet ein Auswahlfenster, wenn mehr als 1 Schnittstelle gefunden worden sind
# und der defaultport in der Liste nicht enthalten ist.

from tkinter import *
from tkinter import ttk
import time

# ------------------------------------------------------------------------------
class SerDialog(object):
    """ Ein Dialog zur Auswahl einer Seriellen Schnittstelle """
    def __init__(self, portlist):
        self.portlist = portlist

        self.top = Tk()
        self.top.title("open_serport()")
        self.top.protocol('WM_DELETE_WINDOW', self._quit)
        self.top.bind('<Escape>', self._quit)
        self.top.resizable(width = False, height = False) # Größe nicht veränderbar

        self.top.bell()

        self.frm = ttk.LabelFrame(self.top, text = "Auswahl Serielle Schnittstelle")
        self.frm.grid(column = 0, row = 0, padx = 5, pady = 5, sticky = "NSEW")

        self.combo = ttk.Combobox(self.frm, values=self.portlist,
                state="readonly", height=2, width = 6)
        self.combo.set(self.portlist[0])
        self.combo.grid(column = 0, row = 1,padx = 5, pady = 5, sticky = "nswe")
        self.combo.bind("<<ComboboxSelected>>", self._change_selection)

        self.lbl = ttk.Label(self.frm, text = "COM-Port auswählen:")
        self.lbl.grid(column = 0, row = 0, columnspan = 2, padx = 5, pady = 5, sticky = "NSEW")

        self.btn1 = ttk.Button(self.frm, text = "Übernehmen",  width = 14, command = self.übernehmen)
        self.btn1.grid(column = 2, row = 1, padx = 5, pady = 5, sticky = "NSEW")
        self.btn1.bind('<Return>', self.übernehmen)
        self.btn1.focus_set()

        self.btn2 = ttk.Button(self.frm, text = "Beenden", width = 14, command = self._quit)
        self.btn2.grid(column = 3, row = 1, padx = 5, pady = 5, sticky = "NSEW")
        self.btn2.bind('<Return>', self._quit)


    def start(self):
        # Vorgabe für die Auswahl ist das erste Element der Liste
        # dies wird zurückgegeben, wenn keine andere Auswahl getroffen wird
        self.selection = self.portlist[0]
        self.top.mainloop()
        return self.selection


    def _change_selection(self, event = None):
        # Holt die Bezeichnung des in der Liste gewählten Elementes
       j = event.widget.current()
       self.selection = self.portlist[j]


    def übernehmen(self, e = None):
        self.top.destroy()


    def _quit(self, e = None):
        # Bei einem Abbruch wird None zurückgegeben.
        self.selection = None
        self.top.destroy()


def main():
    s = ComDialog(["COM1", "COM2", "COM3", "COM4"])
    print("Ausgewählt wurde:", s.start())

if __name__ == '__main__':
    main()
