#!/usr/bin/python3
# -*- coding: UTF-8 -*-
#       21.02.2014

# --- [ serport.py ] -----------------------------------------------------------
#
# Öffnet die Serielle Schnittstelle:
#
# - wenn genau eine freie Schnittstelle gefunden wird, wird diese geöffnet
# - werden mehrere gefunden, dann wird der defaultport geöffnet,
#   sofern er in der Liste der freien Schnittstellen enthalten ist.
# - werden mehrere Schnittstllen gefunden, dann wird eine
#   Dialog geöffnet, es kann aus der Liste ausgewählt werden.

import serial
import sys
from serdialog import SerDialog

# ------------------------------------------------------------------------------
def open_serport(start = 1, end = 13, timeout = 0.1):
    """
    Prüft, ob sich die Ports COM1: bis COM13: öffnen lassen
    gibt eine Liste der öffenbaren Ports zurück.
    Falls genau 1 Port gefunden wurde, wird dieser geöffnet.

    Werden mehrere gefunden, dann kann eine Vorgabe unter
    "default_comport" festgelegt werden.

    Oder es wird der Suchbereich offener Ports durch die
    Parameter 'start' und 'end' eingegrenzt:
        open_serport(start = 9, end = 10) findet nur den Port 9

    Rückgabe: Referenz auf die serielle Schnittstelle
    """
    ports = []
    comport = None
    default_comport = ""

# Die Ports können als "COM1" .. "COM10" oder
# über eine Ziffer      0     ..  9 angesprochen werden

    opsys = sys.platform
    if opsys == "win32":
        for i in range(start, end):
            try:
                p  = "COM" + str(i)
                rx = serial.Serial(p)
                rx.close()
                ports.append(p)

            except IOError:
                pass

    elif op.sys == "linux2":
        # Vielleicht arbeiten wir auf einem RaspberryPi ??
        try:
            p = "/dev/ttyAMA0"
            rx = serial.Serial(p)
            rx.close()
            ports.append(p)

        except IOError:
            pass

    else:
        print("OS =", opsys )
        print("quit\n")
        input("beliebige Taste drücken ...")
        exit()

    print("OS: {} - Freie Ports: {}".format(opsys, ports))

    if len(ports) == 0:         # Kein freier Port gefunden
        print("Kein freier Port gefunden !\n")
        input("beliebige Taste drücken ...")
        exit()

    elif len(ports) == 1:     # genau ein freier Port gefunden
        comport = serial.Serial(ports[0], 9600, rtscts = False)
        print("geöffnet :", comport.name)

    else:                   # mehrere freie Ports gefunden
        if default_comport in ports:
            comport = serial.Serial(default_comport, 9600)
            print("geöffnet : ", default_comport)

        else:
            c = SerDialog(ports)
            ser = c.start()

 #           print(ser)
            if ser is not None:
                comport = serial.Serial(ser, 9600)
                print("geöffnet : ", ser)

            else:
                print("Kein COM-Port geöffnet\n")
                exit()
##            print("welcher Port soll geöffnet werden ?")
##            exit()

    comport.timeout = timeout
    return comport


if __name__ == '__main__':
    open_serport()


# --- [ eof ] ------------------------------------------------------------------
